/*
* Copyright (C) 2009 Medsphere Systems Corporation
*
* This program is free software; you can redistribute it and/or modify it
* solely under the terms of the GNU Affero General Public License version 3 as
* published by the Free Software Foundation.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License
* for more details.
* 
* You should have received a copy of the GNU Affero General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses>.
* 
* You can contact Medsphere Systems Corporation headquarters at 1917 Palomar 
* Oaks Way, Suite 200, Carlsbad, CA 92008 or at legal@medsphere.com. 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <security/pam_appl.h>

#include "authdata.h"
#define SYS_RETRY_MAX 5

int myconv(int n, const struct pam_message **msg, struct pam_response **resp, void *data)
{
  struct pam_response *aresp;
  char buf[PAM_MAX_RESP_SIZE];
  int i;
  int len;
  int sysretry;

  authdata *adata = (authdata *) data;
  if (!adata)
    goto fail;

  if (n <= 0 || n > PAM_MAX_NUM_MSG)
    return (PAM_CONV_ERR);
  if ((aresp = calloc(n, sizeof *aresp)) == NULL)
    return (PAM_BUF_ERR);
  for (i = 0; i < n; ++i) {
    aresp[i].resp_retcode = 0;
    aresp[i].resp = NULL;
    switch (msg[i]->msg_style) {
    case PAM_PROMPT_ECHO_OFF:
      aresp[i].resp = strdup(adata->password);
      if (aresp[i].resp == NULL)
        goto fail;
      break;
    case PAM_PROMPT_ECHO_ON:
      aresp[i].resp = strdup(adata->user);
      if (aresp[i].resp == NULL)
        goto fail;
      break;
    case PAM_ERROR_MSG:
      // fall though
    case PAM_TEXT_INFO:
      // do nothing
      break;
    default:
      goto fail;
    }
  }
  *resp = aresp;
  return (PAM_SUCCESS);
 fail:
  for (i = 0; i < n; ++i) {
    if (aresp[i].resp != NULL) {
      memset(aresp[i].resp, 0, strlen(aresp[i].resp));
      free(aresp[i].resp);
    }
  }
  memset(aresp, 0, n * sizeof *aresp);
  *resp = NULL;
  return (PAM_CONV_ERR);
}

